import math

from libre_finance.constants import KRW_INTEREST_TAX, USD_INTEREST_TAX
from libre_finance.database.spec import FinanceData


class LibreFinance:
    def __init__(self, amount, currency='KRW'):
        self.amount = amount
        self.original_amount = amount
        self.currency = currency
        self.total_interest = 0
        # set interest tax
        if currency == 'KRW':
            self.interest_tax = KRW_INTEREST_TAX
        else:
            self.interest_tax = USD_INTEREST_TAX
        self.history = FinanceData

    def calculate_deposit(self, month, annual_interest_rate=0.0, is_tax=True, is_pretty=True):
        # calculate monthly interest
        interest_rate = annual_interest_rate / 12
        for _ in range(month):
            # calculate interest for each month
            interest = math.floor(self.amount * interest_rate / 100)
            if is_tax:
                # calculate interest tax
                interest *= 1 - self.interest_tax / 100
            self.total_interest += interest
        return_interest = math.floor(self.total_interest)
        self.amount += return_interest
        # saving history
        self.history = FinanceData(
            product='deposit',
            origin_amount=self.original_amount,
            month=month,
            amount=self.amount,
            total_interest=self.total_interest,
            interest_rate=annual_interest_rate,
            interest_tax=self.interest_tax,
            is_tax=is_tax,
            currency=self.currency)
        if is_pretty:
            return f'{return_interest:,}'
        else:
            return return_interest

    def calculate_savings(self, month, annual_interest_rate=0.0, monthly_deposit=0, is_tax=True, is_pretty=True):
        # calculate monthly interest
        interest_rate = annual_interest_rate / 12
        self.amount += monthly_deposit
        for _ in range(month):
            # calculate interest for each month
            interest = math.ceil(self.amount * interest_rate / 100)
            if is_tax:
                # calculate interest tax
                interest *= 1 - self.interest_tax / 100
            self.total_interest += math.floor(interest)
            self.amount += monthly_deposit
        return_interest = math.floor(self.total_interest)
        self.amount += return_interest - monthly_deposit
        # saving history
        self.history = FinanceData(
            product='savings',
            origin_amount=self.original_amount,
            month=month,
            amount=self.amount,
            total_interest=self.total_interest,
            interest_rate=annual_interest_rate,
            interest_tax=self.interest_tax,
            is_tax=is_tax,
            currency=self.currency,
            monthly_deposit=monthly_deposit)
        if is_pretty:
            return f'{return_interest:,}'
        else:
            return return_interest



try:
    from sqlalchemy import create_engine, Table, Column, Integer, String, Index, Boolean, Float
    from sqlalchemy.orm import Session
    from sqlalchemy.schema import MetaData
    from sqlalchemy.sql.functions import now
except ImportError:
    raise ImportError("You need to install \"libre-finance[sql]\"")

from libre_finance.database.spec import FinanceData


class MariaDBTable:
    def __init__(self, db_name, user, passwd, host, port, table_name='finance'):
        self.metadata = MetaData()
        self.main_table = Table(
            table_name,
            self.metadata,
            Column("id", Integer, primary_key=True, autoincrement=True),
            Column("product", String(50), nullable=False),
            Column("month", Integer, nullable=False),
            Column("origin_amount", Integer, nullable=False),
            Column("amount", Integer, nullable=False),
            Column("total_interest", Integer, nullable=False),
            Column("interest_rate", Float, nullable=False),
            Column("interest_tax", Float, nullable=False),
            Column("is_tax", Boolean, nullable=False),
            Column("currency", String(3), nullable=False, server_default="KRW"),
            Column("monthly_deposit", Integer, nullable=True),
            Column("created_at", String(20), nullable=False, server_default=now()),
        )
        self.engine = create_engine(
            f"mariadb+pymysql://{user}:{passwd}@{host}:{port}/{db_name}?charset=utf8mb4"
        )

        self.session = Session(bind=self.engine)

    def create_table(self):
        Index(
            "id_idx",
            self.main_table.c.id
        )

        Index(
            "currency_idx",
            self.main_table.c.currency
        )

        self.metadata.create_all(self.engine)

    def insert_data(self, data: FinanceData):
        with self.session as session:
            insert = self.main_table.insert().values(
                product=data.product,
                origin_amount=data.origin_amount,
                month=data.month,
                amount=data.amount,
                total_interest=data.total_interest,
                interest_rate=data.interest_rate,
                interest_tax=data.interest_tax,
                is_tax=data.is_tax,
                currency=data.currency,
                monthly_deposit=data.monthly_deposit,
            )
            session.execute(insert)
            session.commit()



from dataclasses import dataclass


# Define the finance data structure
@dataclass
class FinanceData:
    origin_amount: int
    product: str
    month: int
    amount: int
    total_interest: int
    interest_rate: float
    interest_tax: float
    is_tax: bool
    # monthly_deposit is optional
    monthly_deposit: int = None
    currency: str = "KRW"
